-------------------------------------------------------------------------------
--
-- Copyright (C) 2019
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Author:
-- . Reinier van der Walle
-- Purpose:
-- . Instantiates ta2_unb2b_1GbE_mc component 
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY ta2_unb2b_1GbE_mc_ip_wrapper IS       
  PORT (      
    st_clk             : IN STD_LOGIC;
    st_rst             : IN STD_LOGIC;

    -- eth1g UDP streaming ports
    udp_tx_sosi_data   : OUT STD_LOGIC_VECTOR(39 DOWNTO 0);
    udp_tx_sosi_valid  : OUT STD_LOGIC;
    udp_tx_sosi_sop    : OUT STD_LOGIC;
    udp_tx_sosi_eop    : OUT STD_LOGIC;
    udp_tx_sosi_empty  : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    udp_tx_siso_ready  : IN  STD_LOGIC;  
    udp_tx_siso_xon    : IN  STD_LOGIC; 
 
    udp_rx_sosi_data   : IN  STD_LOGIC_VECTOR(39 DOWNTO 0);
    udp_rx_sosi_valid  : IN  STD_LOGIC;
    udp_rx_sosi_sop    : IN  STD_LOGIC;
    udp_rx_sosi_eop    : IN  STD_LOGIC;
    udp_rx_sosi_empty  : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
    udp_rx_siso_ready  : OUT STD_LOGIC;  
    udp_rx_siso_xon    : OUT STD_LOGIC; 
 
    kernel_clk         : IN  STD_LOGIC; -- Kernel clock (runs the kernel_* I/O below)
    kernel_reset       : IN  STD_LOGIC;

    kernel_src_data    : OUT STD_LOGIC_VECTOR(39 DOWNTO 0); -- RX Data to kernel
    kernel_src_valid   : OUT STD_LOGIC; -- RX data valid signal to kernel
    kernel_src_ready   : IN  STD_LOGIC; -- Flow control from kernel

    kernel_snk_data    : IN  STD_LOGIC_VECTOR(39 DOWNTO 0); -- TX Data from kernel
    kernel_snk_valid   : IN  STD_LOGIC; -- TX data valid signal from kernel
    kernel_snk_ready   : OUT STD_LOGIC   -- Flow control towards kernel
  );
END ta2_unb2b_1GbE_mc_ip_wrapper;


ARCHITECTURE str OF ta2_unb2b_1GbE_mc_ip_wrapper IS
  ----------------------------------------------------------------------------
  -- ta2_unb2b_1GbE_mc Component
  ----------------------------------------------------------------------------
  COMPONENT ta2_unb2b_1GbE_mc IS
  PORT (
    st_clk             : IN STD_LOGIC;
    st_rst             : IN STD_LOGIC;

    -- eth1g UDP streaming ports
    udp_tx_sosi_data   : OUT STD_LOGIC_VECTOR(39 DOWNTO 0);
    udp_tx_sosi_valid  : OUT STD_LOGIC;
    udp_tx_sosi_sop    : OUT STD_LOGIC;
    udp_tx_sosi_eop    : OUT STD_LOGIC;
    udp_tx_sosi_empty  : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    udp_tx_siso_ready  : IN  STD_LOGIC;  
    udp_tx_siso_xon    : IN  STD_LOGIC; 
 
    udp_rx_sosi_data   : IN  STD_LOGIC_VECTOR(39 DOWNTO 0);
    udp_rx_sosi_valid  : IN  STD_LOGIC;
    udp_rx_sosi_sop    : IN  STD_LOGIC;
    udp_rx_sosi_eop    : IN  STD_LOGIC;
    udp_rx_sosi_empty  : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
    udp_rx_siso_ready  : OUT STD_LOGIC;  
    udp_rx_siso_xon    : OUT STD_LOGIC;  

    kernel_clk         : IN  STD_LOGIC; -- Kernel clock (runs the kernel_* I/O below)
    kernel_reset       : IN  STD_LOGIC;

    kernel_src_data    : OUT STD_LOGIC_VECTOR(39 DOWNTO 0); -- RX Data to kernel
    kernel_src_valid   : OUT STD_LOGIC; -- RX data valid signal to kernel
    kernel_src_ready   : IN  STD_LOGIC; -- Flow control from kernel

    kernel_snk_data    : IN  STD_LOGIC_VECTOR(39 DOWNTO 0); -- TX Data from kernel
    kernel_snk_valid   : IN  STD_LOGIC; -- TX data valid signal from kernel
    kernel_snk_ready   : OUT STD_LOGIC   -- Flow control towards kernel
  );
  END COMPONENT ta2_unb2b_1GbE_mc;

BEGIN

  u_ta2_unb2b_1GbE_mc : ta2_unb2b_1GbE_mc 
    PORT MAP (
      st_clk             => st_clk,
      st_rst             => st_rst,    

      udp_tx_sosi_data   => udp_tx_sosi_data,
      udp_tx_sosi_valid  => udp_tx_sosi_valid,
      udp_tx_sosi_sop    => udp_tx_sosi_sop,
      udp_tx_sosi_eop    => udp_tx_sosi_eop,
      udp_tx_sosi_empty  => udp_tx_sosi_empty,
      udp_tx_siso_ready  => udp_tx_siso_ready,
      udp_tx_siso_xon    => udp_tx_siso_xon,
                                             
      udp_rx_sosi_data   => udp_rx_sosi_data,
      udp_rx_sosi_valid  => udp_rx_sosi_valid,
      udp_rx_sosi_sop    => udp_rx_sosi_sop,
      udp_rx_sosi_eop    => udp_rx_sosi_eop,
      udp_rx_sosi_empty  => udp_rx_sosi_empty,
      udp_rx_siso_ready  => udp_rx_siso_ready,
      udp_rx_siso_xon    => udp_rx_siso_xon,
 
      kernel_clk         => kernel_clk,
      kernel_reset       => kernel_reset,
      kernel_src_data    => kernel_src_data ,
      kernel_src_valid   => kernel_src_valid,
      kernel_src_ready   => kernel_src_ready,
      kernel_snk_data    => kernel_snk_data,
      kernel_snk_valid   => kernel_snk_valid,
      kernel_snk_ready   => kernel_snk_ready 
    );

END str;
