-------------------------------------------------------------------------------
--
-- Copyright (C) 2019
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Author:
-- . Reinier van der Walle
-- Purpose:
-- . Instantiates ta2_unb2b_10GbE component 
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY ta2_unb2b_jesd204b_ip_wrapper IS       
  PORT (      
    config_clk       : IN  STD_LOGIC; -- 100MHz clk for reconfig block and status interface
    config_reset     : IN  STD_LOGIC;

    -- MM Control
    jesd204b_mosi_address     : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
    jesd204b_mosi_wrdata      : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    jesd204b_mosi_wr          : IN  STD_LOGIC;
    jesd204b_mosi_rd          : IN  STD_LOGIC;
    jesd204b_miso_rddata      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    jesd204b_miso_waitrequest : OUT STD_LOGIC;

    -- JESD204B external signals
    jesd204b_refclk       : IN STD_LOGIC := '0';                             -- Reference clock. For AD9683 use 200MHz direct from clock reference pin
    jesd204b_sysref       : IN STD_LOGIC := '0';                             -- SYSREF should drive ADC and FPGA with correct phase wrt jesd204b_device_clk 
    jesd204b_sync_n_arr   : OUT STD_LOGIC_VECTOR(0 DOWNTO 0); -- output to control ADC initialization/syncronization phase
 
    serial_rx_arr         : IN  STD_LOGIC_VECTOR(0 DOWNTO 0);

    kernel_clk       : IN  STD_LOGIC; -- Kernel clock (runs the kernel_* I/O below)
    kernel_reset     : IN  STD_LOGIC;

    kernel_src_data  : OUT STD_LOGIC_VECTOR(15 DOWNTO 0); -- RX Data to kernel
    kernel_src_valid : OUT STD_LOGIC; -- RX data valid signal to kernel
    kernel_src_ready : IN  STD_LOGIC -- Flow control from kernel

  );
END ta2_unb2b_jesd204b_ip_wrapper;


ARCHITECTURE str OF ta2_unb2b_jesd204b_ip_wrapper IS
  ----------------------------------------------------------------------------
  -- ta2_unb2b_ Component
  ----------------------------------------------------------------------------
  COMPONENT ta2_unb2b_jesd204b IS       
    PORT (      
      config_clk       : IN  STD_LOGIC; -- 100MHz clk for reconfig block and status interface
      config_reset     : IN  STD_LOGIC;
  
      -- MM Control
      jesd204b_mosi_address     : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
      jesd204b_mosi_wrdata      : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
      jesd204b_mosi_wr          : IN  STD_LOGIC;
      jesd204b_mosi_rd          : IN  STD_LOGIC;
      jesd204b_miso_rddata      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
      jesd204b_miso_waitrequest : OUT STD_LOGIC;
  
      -- JESD204B external signals
      jesd204b_refclk       : IN STD_LOGIC := '0';                             -- Reference clock. For AD9683 use 200MHz direct from clock reference pin
      jesd204b_sysref       : IN STD_LOGIC := '0';                             -- SYSREF should drive ADC and FPGA with correct phase wrt jesd204b_device_clk 
      jesd204b_sync_n_arr   : OUT STD_LOGIC_VECTOR(0 DOWNTO 0); -- output to control ADC initialization/syncronization phase
   
      serial_rx_arr         : IN  STD_LOGIC_VECTOR(0 DOWNTO 0);
  
      kernel_clk       : IN  STD_LOGIC; -- Kernel clock (runs the kernel_* I/O below)
      kernel_reset     : IN  STD_LOGIC;
  
      kernel_src_data  : OUT STD_LOGIC_VECTOR(15 DOWNTO 0); -- RX Data to kernel
      kernel_src_valid : OUT STD_LOGIC; -- RX data valid signal to kernel
      kernel_src_ready : IN  STD_LOGIC -- Flow control from kernel
  
    );
  END COMPONENT ta2_unb2b_jesd204b;



BEGIN

  u_ta2_unb2b_jesd204b : ta2_unb2b_jesd204b
    PORT MAP (      
      config_clk                => config_clk, 
      config_reset              => config_reset, 
      jesd204b_mosi_address     => jesd204b_mosi_address,    
      jesd204b_mosi_wrdata      => jesd204b_mosi_wrdata,     
      jesd204b_mosi_wr          => jesd204b_mosi_wr,         
      jesd204b_mosi_rd          => jesd204b_mosi_rd,         
      jesd204b_miso_rddata      => jesd204b_miso_rddata,     
      jesd204b_miso_waitrequest => jesd204b_miso_waitrequest,
      jesd204b_refclk           => jesd204b_refclk,   
      jesd204b_sysref           => jesd204b_sysref,    
      jesd204b_sync_n_arr       => jesd204b_sync_n_arr,    
      serial_rx_arr             => serial_rx_arr,   
      kernel_clk                => kernel_clk,   
      kernel_reset              => kernel_reset,    
      kernel_src_data           => kernel_src_data,   
      kernel_src_valid          => kernel_src_valid,    
      kernel_src_ready          => kernel_src_ready       
    );

END str;

